import os
import re
import sys
import glob
import optparse

import simplejson

from lazr.js.build import SRC_DIR


DETAILS_RE = re.compile(
    "YUI\.add\([\'\"\s]*([^\'\"]*)[\'\"\s]*,.*?function.*?"
    "[\'\"\s]*[0-9\.]*[\'\"\s]*"
    "({[^{}]*(use|requires|optional|after|"
    "supersedes|omit|skinnable)[^{}]*})\s*\);", re.M | re.S)


def extract_metadata(src):
    """Extract metadata about an YUI module, given it's source."""
    metadata = []
    for entry in DETAILS_RE.finditer(src):
        name, details, ignore = entry.groups()
        details = simplejson.loads(details)
        details["name"] = name
        metadata.append(details)
    return metadata


class Builder:

    def __init__(self, name, src_dir=SRC_DIR, output=None,
                 prefix="", exclude_regex=None, ext=True):
        """Create a new Builder.

        :param name: The name of the global variable name that will
            contain the modules configuration.
        :param src_dir: The directory containing the source files.
        :param output: The output filename for the module metadata.
        :param prefix: A prefix to be added to the relative path.
        :param exclude_regex: A regex that will exclude file
            paths from the final rollup.  -min and -debug versions
            will still be built.
        :param ext: Default value for the 'ext' setting. Set to
            'False' to use modules with a local combo loader.
        """
        self.name = name
        self.output = output
        self.prefix = prefix
        self.src_dir = src_dir
        self.exclude_regex = exclude_regex
        self.ext = ext

    def log(self, msg):
        sys.stdout.write(msg + '\n')

    def fail(self, msg):
        """An error was encountered, abort build."""
        sys.stderr.write(msg + '\n')
        sys.exit(1)

    def file_is_excluded(self, filepath):
        """Is the given file path excluded from the rollup process?"""
        if not self.exclude_regex:
            # Include everything.
            return False

        return re.search(self.exclude_regex, filepath)

    def generate_metadata(self, fnames, root, var_name, out):
        """Extract metadata from a group of files and write it out."""
        metadata = []

        for fname in fnames:
            self.log("Extracting metadata from '%s'" % fname)
            data = open(fname, "r").read()
            meta = extract_metadata(data)
            prefix = ""
            if self.prefix and not prefix.endswith("/"):
                prefix = self.prefix + "/"
            for entry in meta:
                entry["path"] = (
                    prefix + fname.replace(root + os.path.sep, ""))
                entry["type"] = "js"
                entry["ext"] = self.ext
                if entry.get("skinnable"):
                    # Generate a skin definition, since YUI assumes
                    # that the path starts with the module name, and
                    # that breaks with our structure.
                    css = os.path.join(os.path.dirname(entry["path"]),
                                       "assets", "skins", "sam",
                                       entry["name"] + ".css")
                    metadata.append({"name": "skin-sam-%s" % entry["name"],
                                     "after": ["cssreset", "cssfonts",
                                               "cssgrids", "cssreset-context",
                                               "cssfonts-context",
                                               "cssgrids-context"],
                                     "type": "css",
                                     "ext": self.ext,
                                     "path": css})
            metadata.extend(meta)

        modules = {}
        for meta in metadata:
            name = meta.pop("name")
            modules[name] = meta

        open(out, "w").write("var %s = %s;" %
                             (var_name, simplejson.dumps(modules)))

    def do_build(self):
        included_files = []

        for root, dirnames, fnames in os.walk(self.src_dir):
            for fname in glob.glob(os.path.join(root, '*.js')):
                if not self.file_is_excluded(fname):
                    included_files.append(fname)

        self.generate_metadata(included_files, self.src_dir,
                               self.name, self.output)


def get_options():
    """Parse the command line options."""
    parser = optparse.OptionParser(
        usage="%prog [options]",
        description=(
            "Create YUI module metadata from extension modules. "
            ))
    parser.add_option(
        '-n', '--name', dest='name', default='LAZR_MODULES',
        help=('The global variable name used to hold the modules config.'))
    parser.add_option(
        '-o', '--output', dest='output',
        help=('The output filename for the module metadata.'))
    parser.add_option(
        '-p', '--prefix', dest='prefix',
        help=('A prefix to be added to the relative path.'))
    parser.add_option(
        '-s', '--srcdir', dest='src_dir', default=SRC_DIR,
        help=('The directory containing the src files.'))
    parser.add_option(
        '-e', '--ext', dest='ext', default=False,
        action="store_true",
        help=('Default value for the "ext" configuration setting.'))
    parser.add_option(
        '-x', '--exclude', dest='exclude_regex',
        default=None, metavar='REGEX',
        help=('Exclude any files that match the given regular expressions.'))
    return parser.parse_args()


def main():
   options, args = get_options()
   Builder(
       name=options.name,
       src_dir=os.path.abspath(options.src_dir),
       output=options.output,
       prefix=options.prefix,
       exclude_regex=options.exclude_regex,
       ext=options.ext,
       ).do_build()
